@@ -359,11 +359,7 @@ gupnp_context_finalize (GObject *object)
 
         context = GUPNP_CONTEXT (object);
 
-        if (context->priv->default_language) {
-                g_free (context->priv->default_language);
-                context->priv->default_language = NULL;
-        }
-
+        g_free (context->priv->default_language);
         g_free (context->priv->server_url);
 
         /* Call super */
@@ -502,6 +498,8 @@ gupnp_context_class_init (GUPnPContextClass *klass)
          * The content of the Content-Language header id the client
          * sends Accept-Language and no language-specific pages to serve
          * exist. The property defaults to 'en'.
+         *
+         * Since: 0.17.0
          **/
         g_object_class_install_property
                 (object_class,
@@ -516,6 +514,13 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                       G_PARAM_STATIC_NICK |
                                       G_PARAM_STATIC_BLURB));
 
+        /**
+         * GUPnPContext:acl:
+         *
+         * An access control list.
+         *
+         * Since: 0.20.11
+         */
         g_object_class_install_property
                 (object_class,
                  PROP_ACL,
@@ -536,6 +541,8 @@ gupnp_context_class_init (GUPnPContextClass *klass)
  *
  * Return value: (transfer none): The #SoupSession used by GUPnP. Do not unref
  * this when finished.
+ *
+ * Since: 0.12.3
  **/
 SoupSession *
 gupnp_context_get_session (GUPnPContext *context)
@@ -749,9 +756,7 @@ host_path_data_set_language (HostPathData *data, const char *language)
 
         data->default_language = g_strdup (language);
 
-
-        if (old_language != NULL)
-                g_free (old_language);
+        g_free (old_language);
 }
 
 /**
@@ -765,6 +770,8 @@ host_path_data_set_language (HostPathData *data, const char *language)
  * is required to send a Content-Language header in return. If there are
  * no files hosted in languages which match the requested ones the
  * Content-Language header is set to this value. The default value is "en".
+ *
+ * Since: 0.17.0
  */
 void
 gupnp_context_set_default_language (GUPnPContext *context,
@@ -785,8 +792,7 @@ gupnp_context_set_default_language (GUPnPContext *context,
                         (GFunc) host_path_data_set_language,
                         (gpointer) language);
 
-        if (old_language != NULL)
-                g_free (old_language);
+        g_free (old_language);
 }
 
 /**
@@ -797,6 +803,8 @@ gupnp_context_set_default_language (GUPnPContext *context,
  *
  * Returns: (transfer none): The default content of the Content-Language
  * header.
+ *
+ * Since: 0.17.0
  */
 const char *
 gupnp_context_get_default_language (GUPnPContext *context)
@@ -1124,10 +1132,7 @@ host_path_handler (G_GNUC_UNUSED SoupServer        *server,
         g_free (path_to_open);
         g_free (local_path);
 
-        while (orig_locales) {
-                g_free (orig_locales->data);
-                orig_locales = g_list_delete_link (orig_locales, orig_locales);
-        }
+        g_list_free_full (orig_locales, g_free);
 }
 
 static UserAgent *
@@ -1176,17 +1181,8 @@ host_path_data_free (HostPathData *path_data)
         g_free (path_data->server_path);
         g_free (path_data->default_language);
 
-        while (path_data->user_agents) {
-                UserAgent *agent;
-
-                agent = path_data->user_agents->data;
-
-                user_agent_free (agent);
-
-                path_data->user_agents = g_list_delete_link (
-                                path_data->user_agents,
-                                path_data->user_agents);
-        }
+        g_list_free_full (path_data->user_agents,
+                          (GDestroyNotify) user_agent_free);
 
         g_slice_free (HostPathData, path_data);
 }
@@ -1249,6 +1245,8 @@ path_compare_func (HostPathData *path_data,
  * path @server_path must already be hosted by @context.
  *
  * Return value: %TRUE on success, %FALSE otherwise.
+ *
+ * Since: 0.13.3
  **/
 gboolean
 gupnp_context_host_path_for_agent (GUPnPContext *context,
@@ -1321,6 +1319,8 @@ gupnp_context_unhost_path (GUPnPContext *context,
  *
  * Returns:(transfer none): The access control list associated with this context or %NULL
  * if no acl is set.
+ *
+ * Since: 0.20.11
  **/
 GUPnPAcl *
 gupnp_context_get_acl (GUPnPContext *context)
@@ -1335,6 +1335,8 @@ gupnp_context_get_acl (GUPnPContext *context)
  * @context: A #GUPnPContext
  * @acl: (allow-none): The new access control list or %NULL to remove the
  * current list.
+ *
+ * Since: 0.20.11
  **/
 void
 gupnp_context_set_acl (GUPnPContext *context, GUPnPAcl *acl)
@@ -1391,6 +1393,10 @@ gupnp_acl_server_handler (SoupServer *server,
                 g_object_get (handler->service,
                               "root-device", &device,
                               NULL);
+
+                if (device != NULL) {
+                        g_object_unref (device);
+                }
         }
 
         agent = soup_message_headers_get_one (msg->request_headers,
@@ -1444,6 +1450,7 @@ gupnp_acl_server_handler (SoupServer *server,
  *
  * Add a #SoupServerCallback to the #GUPnPContext<!-- -->'s #SoupServer.
  *
+ * Since: 0.20.11
  */
 void
 gupnp_context_add_server_handler (GUPnPContext *context,
@@ -1494,6 +1501,7 @@ _gupnp_context_add_server_handler_with_data (GUPnPContext *context,
  *
  * Add a #SoupServerCallback to the #GUPnPContext<!-- -->'s #SoupServer.
  *
+ * Since: 0.20.11
  */
 void
 gupnp_context_remove_server_handler (GUPnPContext *context, const char *path)
