@@ -95,10 +95,8 @@ gupnp_service_state_variable_info_free
                 g_value_unset (&variable->maximum);
                 g_value_unset (&variable->step);
         }
-        g_list_foreach (variable->allowed_values,
-                         (GFunc) g_free,
-                         NULL);
-        g_list_free (variable->allowed_values);
+
+        g_list_free_full (variable->allowed_values, g_free);
 
         g_slice_free (GUPnPServiceStateVariableInfo, variable);
 }
@@ -164,12 +162,8 @@ gupnp_service_action_info_free (GUPnPServiceActionInfo *action_info)
         GList *iter;
 
         g_free (action_info->name);
-
-        for (iter = action_info->arguments; iter; iter = iter->next) {
-                gupnp_service_action_arg_info_free (
-                                (GUPnPServiceActionArgInfo *) iter->data);
-        }
-        g_list_free (action_info->arguments);
+        g_list_free_full (action_info->arguments,
+                          (GDestroyNotify) gupnp_service_action_arg_info_free);
         g_slice_free (GUPnPServiceActionInfo, action_info);
 }
 
@@ -180,23 +174,19 @@ gupnp_service_introspection_finalize (GObject *object)
 
         introspection = GUPNP_SERVICE_INTROSPECTION (object);
 
-        if (introspection->priv->variables) {
-                g_list_foreach (introspection->priv->variables,
-                                (GFunc) gupnp_service_state_variable_info_free,
-                                NULL);
-                g_list_free (introspection->priv->variables);
-        }
+        g_list_free_full (introspection->priv->variables,
+                          (GDestroyNotify) gupnp_service_state_variable_info_free);
 
-        if (introspection->priv->actions) {
-                g_list_foreach (introspection->priv->actions,
-                                (GFunc) gupnp_service_action_info_free,
-                                NULL);
-                g_list_free (introspection->priv->actions);
-        }
+        g_list_free_full (introspection->priv->actions,
+                          (GDestroyNotify) gupnp_service_action_info_free);
 
+        /* Contents don't need to be freed, they were owned by priv->variables
+         */
         if (introspection->priv->variable_names)
                 g_list_free (introspection->priv->variable_names);
 
+        /* Contents don't need to be freed, they were owned by priv->actions
+         */
         if (introspection->priv->action_names)
                 g_list_free (introspection->priv->action_names);
 }
@@ -445,7 +435,7 @@ set_variable_type (GUPnPServiceStateVariableInfo *variable,
         }
 
         if (type == G_TYPE_INVALID) {
-                g_warning ("Unkown type '%s' in the SCPD", data_type);
+                g_warning ("Unknown type '%s' in the SCPD", data_type);
                 return FALSE;
         }
 
@@ -851,7 +841,7 @@ state_variable_search_func (GUPnPServiceStateVariableInfo *variable,
 /**
  * gupnp_service_introspection_get_state_variable:
  * @introspection: A #GUPnPServiceIntrospection
- * @variable_name: The name of the variable to retreive
+ * @variable_name: The name of the variable to retrieve
  *
  * Returns the state variable by the name @variable_name in this service.
  *
@@ -888,7 +878,7 @@ action_search_func (GUPnPServiceActionInfo *action,
 /**
  * gupnp_service_introspection_get_action:
  * @introspection: A #GUPnPServiceIntrospection
- * @action_name: The name of the action to retreive
+ * @action_name: The name of the action to retrieve
  *
  * Returns the action by the name @action_name in this service.
  *
